@@ -30,6 +30,7 @@ pub unsafe trait Data : Sized {
     #[doc(hidden)]
     // This method is only used for debugging
     fn _data_slice(&self) -> &[Self::Elem];
+    private_decl!{}
 }
 
 /// Array representation trait.
@@ -73,8 +74,9 @@ pub unsafe trait DataClone : Data {
 unsafe impl<A> Data for OwnedRcRepr<A> {
     type Elem = A;
     fn _data_slice(&self) -> &[A] {
-        self
+        &self.0
     }
+    private_impl!{}
 }
 
 // NOTE: Copy on write
@@ -85,10 +87,10 @@ unsafe impl<A> DataMut for OwnedRcRepr<A>
         where Self: Sized,
               D: Dimension
     {
-        if Rc::get_mut(&mut self_.data).is_some() {
+        if Rc::get_mut(&mut self_.data.0).is_some() {
             return;
         }
-        if self_.dim.size() <= self_.data.len() / 2 {
+        if self_.dim.size() <= self_.data.0.len() / 2 {
             // Create a new vec if the current view is less than half of
             // backing data.
             unsafe {
@@ -99,18 +101,19 @@ unsafe impl<A> DataMut for OwnedRcRepr<A>
             }
             return;
         }
+        let rcvec = &mut self_.data.0;
         let a_size = mem::size_of::<A>() as isize;
         let our_off = if a_size != 0 {
-            (self_.ptr as isize - self_.data.as_ptr() as isize) / a_size
+            (self_.ptr as isize - rcvec.as_ptr() as isize) / a_size
         } else { 0 };
-        let rvec = Rc::make_mut(&mut self_.data);
+        let rvec = Rc::make_mut(rcvec);
         unsafe {
             self_.ptr = rvec.as_mut_ptr().offset(our_off);
         }
     }
 
     fn is_unique(&mut self) -> bool {
-        Rc::get_mut(self).is_some()
+        Rc::get_mut(&mut self.0).is_some()
     }
 }
 
@@ -124,8 +127,9 @@ unsafe impl<A> DataClone for OwnedRcRepr<A> {
 unsafe impl<A> Data for OwnedRepr<A> {
     type Elem = A;
     fn _data_slice(&self) -> &[A] {
-        self
+        &self.0
     }
+    private_impl!{}
 }
 
 unsafe impl<A> DataMut for OwnedRepr<A> { }
@@ -135,9 +139,9 @@ unsafe impl<A> DataClone for OwnedRepr<A>
 {
     unsafe fn clone_with_ptr(&self, ptr: *mut Self::Elem) -> (Self, *mut Self::Elem) {
         let mut u = self.clone();
-        let mut new_ptr = u.as_mut_ptr();
+        let mut new_ptr = u.0.as_mut_ptr();
         if size_of::<A>() != 0 {
-            let our_off = (ptr as isize - self.as_ptr() as isize) /
+            let our_off = (ptr as isize - self.0.as_ptr() as isize) /
                           mem::size_of::<A>() as isize;
             new_ptr = new_ptr.offset(our_off);
         }
@@ -146,13 +150,13 @@ unsafe impl<A> DataClone for OwnedRepr<A>
 
     unsafe fn clone_from_with_ptr(&mut self, other: &Self, ptr: *mut Self::Elem) -> *mut Self::Elem {
         let our_off = if size_of::<A>() != 0 {
-            (ptr as isize - other.as_ptr() as isize) /
+            (ptr as isize - other.0.as_ptr() as isize) /
                           mem::size_of::<A>() as isize
         } else {
             0
         };
-        self.clone_from(other);
-        self.as_mut_ptr().offset(our_off)
+        self.0.clone_from(&other.0);
+        self.0.as_mut_ptr().offset(our_off)
     }
 }
 
@@ -161,6 +165,7 @@ unsafe impl<'a, A> Data for ViewRepr<&'a A> {
     fn _data_slice(&self) -> &[A] {
         &[]
     }
+    private_impl!{}
 }
 
 unsafe impl<'a, A> DataClone for ViewRepr<&'a A> {
@@ -174,6 +179,7 @@ unsafe impl<'a, A> Data for ViewRepr<&'a mut A> {
     fn _data_slice(&self) -> &[A] {
         &[]
     }
+    private_impl!{}
 }
 
 unsafe impl<'a, A> DataMut for ViewRepr<&'a mut A> { }
@@ -202,16 +208,16 @@ unsafe impl<'a, A> DataShared for ViewRepr<&'a A> {}
 
 unsafe impl<A> DataOwned for OwnedRepr<A> {
     fn new(elements: Vec<A>) -> Self {
-        elements
+        OwnedRepr(elements)
     }
     fn into_shared(self) -> OwnedRcRepr<A> {
-        Rc::new(self)
+        OwnedRcRepr(Rc::new(self.0))
     }
 }
 
 unsafe impl<A> DataOwned for OwnedRcRepr<A> {
     fn new(elements: Vec<A>) -> Self {
-        Rc::new(elements)
+        OwnedRcRepr(Rc::new(elements))
     }
     fn into_shared(self) -> OwnedRcRepr<A> {
         self
